import { AbstractLogger } from "../logging/AbstractLogger";
import { ClaireApp } from "../system/ClaireApp";

export interface ISetConfigurations<C, E> {
    setConfiguration(environment: string, configurationResolver?: Promise<C>): ISetLogger<C, E>;
}

export interface ISetLogger<C, E> {
    setLogger(loggerResolver: (config: C) => AbstractLogger): E;
}

export abstract class ClaireAppBuilderTerminal<C, A extends ClaireApp<C>> {
    public environment?: string;
    public configurationResolver?: Promise<C>;
    public loggerResolver: (config: C) => AbstractLogger;

    public abstract build(): A;
}

export abstract class ClaireAppBuilder<C, A extends ClaireApp<C>, E, K extends ClaireAppBuilderTerminal<C, A> & E>
    implements ISetConfigurations<C, E>, ISetLogger<C, E> {
    protected builderTerminal: K;

    public constructor(builderTerminal: K) {
        this.builderTerminal = builderTerminal;
    }

    public setConfiguration(environment: string, configurationResolver: Promise<C>): ISetLogger<C, E> {
        this.builderTerminal.environment = environment;
        this.builderTerminal.configurationResolver = configurationResolver;
        return this;
    }

    public setLogger(loggerResolver: (config: C) => AbstractLogger): E {
        this.builderTerminal.loggerResolver = loggerResolver;
        return this.builderTerminal;
    }
}
