import { getInjectionRegistry } from "./injection";
import { IStartStop } from "../system/IStartStop";
import { initObjectMetadata, initFieldMetadata } from "../common/metadata";
import { InitableMetadata } from "../common/InitableMetadata";

export const Inject = (constructor?: any) => (prototype: any, propertyKey: string) => {
    const t = Reflect.getMetadata("design:type", prototype, propertyKey);
    const meta = initFieldMetadata(prototype, propertyKey);
    meta.injecteeSuperClass = constructor || t;
    getInjectionRegistry().registerTarget(prototype, propertyKey, meta.injecteeSuperClass);
};

export const Injectable = (config?: { environment?: string[] }) => (constructor: { new (): any }) => {
    getInjectionRegistry().registerInjectable(constructor, config?.environment);
};

export const Initable = () => <T extends { new (...args: any[]): IStartStop }>(constructor: T) => {
    const metadata = initObjectMetadata<InitableMetadata>(constructor.prototype);
    metadata.initable = true;
};
