import { OrderDirection } from "./OrderDirection";
import { Integral, DTO, Required, Vector, NumberRange } from "../dto/decorator";

//-- create
@DTO({ strict: true })
export class CreateManyRequestBody<T> {
    @Required()
    @Vector({ minLength: 1 })
    records: Partial<T>[];
}

@DTO({ strict: true })
export class CreateManyResponseBody<T> {
    @Required()
    records: Partial<T>[];
}

//-- read

@DTO({ strict: false })
export class IdParam {
    @Required()
    @Integral()
    id: number;
}

@DTO({ strict: false })
export class GetManyQueries<T> {
    @Required(false)
    fields?: { [k in keyof T]?: any };

    @Required(false)
    search?: string;

    @Required(false)
    @Vector({ minLength: 1 })
    projection?: (keyof T)[];

    @Required(false)
    @Integral()
    @NumberRange({ min: 1 })
    limit?: number;

    @Required(false)
    @Integral()
    @NumberRange({ min: 1 })
    page?: number;

    @Required(false)
    @Vector({ minLength: 1 })
    order?: { [k in keyof T]?: OrderDirection }[];
}

@DTO({ strict: true })
export class GetManyResponseBody<T> {
    @Required()
    total: number;

    @Required()
    @Vector()
    records: T[];
}

//-- update & delete
@DTO({ strict: false })
export class UpdateManyQueries<T> {
    @Required(false)
    fields?: { [key in keyof T]?: any };

    //-- return array of ids of modified instances
    @Required(false)
    returning?: boolean;
}

@DTO({ strict: true })
export class UpdateManyResponse {
    @Required()
    //-- array of ids of modified instances
    modified: number[];
}

@DTO({ strict: true })
export class UpdateManyBody<T> {
    @Required()
    update: Partial<T>;
}
