import { LogLevel } from "./LogLevel";
import { ILogContentSerializer } from "./ILogContentSerializer";
import { ILogMedium } from "./media/ILogMedium";

export abstract class AbstractLogger {
    protected limit: LogLevel = LogLevel.DEBUG;
    protected logMedia: ILogMedium[] = [];
    protected contentSerializer: ILogContentSerializer;

    public constructor(limit: LogLevel, logMedia: ILogMedium[], contentSerializer: ILogContentSerializer) {
        this.limit = limit;
        this.logMedia = logMedia;
        this.contentSerializer = contentSerializer;
    }

    private writeToMedia(level: LogLevel, content: any[]) {
        const logContent = this.contentSerializer.serialize(level, content);
        for (const medium of this.logMedia) {
            medium.write(level, logContent);
        }
    }

    public getLogLimit(): LogLevel {
        return this.limit;
    }

    public debug(...content: any[]): void {
        if (![LogLevel.DEBUG].includes(this.limit)) {
            return;
        }
        return this.writeToMedia(LogLevel.DEBUG, content);
    }

    public info(...content: any[]): void {
        if (![LogLevel.DEBUG, LogLevel.INFO].includes(this.limit)) {
            return;
        }
        return this.writeToMedia(LogLevel.INFO, content);
    }

    public warn(...content: any[]): void {
        if (![LogLevel.DEBUG, LogLevel.INFO, LogLevel.WARN].includes(this.limit)) {
            return;
        }
        return this.writeToMedia(LogLevel.WARN, content);
    }

    public error(...content: any[]): void {
        return this.writeToMedia(LogLevel.ERROR, content);
    }
}
