import { AbstractLogger } from "./AbstractLogger";
import { ILogContentSerializer } from "./ILogContentSerializer";
import { LogLevel } from "./LogLevel";
import { ILogMedium } from "./media/ILogMedium";

class DefaultLogContentSerializer implements ILogContentSerializer {
    public serialize(level: LogLevel, content: any[]): string {
        let logContent = "";

        for (let i = 0; i < content.length; i += 1) {
            logContent += typeof content[i] === "string" ? content[i] : JSON.stringify(content[i]);
            if (i < content.length - 1) {
                logContent += "\n";
            }
        }

        switch (level) {
            case LogLevel.DEBUG:
                return "[DEBG]: " + logContent;
            case LogLevel.INFO:
                return "[INFO]: " + logContent;
            case LogLevel.WARN:
                return "[WARN]: " + logContent;
            case LogLevel.ERROR:
                return "[ERRR]: " + logContent;
        }
    }
}

export class DefaultLogger extends AbstractLogger {
    public constructor(logLimit: LogLevel, logMedia: ILogMedium[]) {
        super(logLimit, logMedia, new DefaultLogContentSerializer());
    }
}
