import { getInjectionRegistry } from "../di/injection";
import { AbstractLogger } from "../logging/AbstractLogger";
import { AbstractConfiguration } from "./AbstractConfiguration";

export class ClaireApp<C extends AbstractConfiguration> {
    private readonly configurationResolver?: Promise<C>;
    private readonly loggerResolver: (config: any) => AbstractLogger;

    //-- protected
    protected readonly injectionRegistry = getInjectionRegistry();
    protected readonly environment?: string;
    protected configuration: any;
    protected logger: AbstractLogger;

    public constructor(config: { environment?: string; configurationResolver?: Promise<C>; loggerResolver: (config: C) => AbstractLogger }) {
        this.environment = config.environment;
        this.configurationResolver = config.configurationResolver;
        this.loggerResolver = config.loggerResolver;
    }

    protected async boot(): Promise<void> {
        //-- resolve configuration
        this.configuration = this.configurationResolver && this.environment ? await this.configurationResolver : {};
        this.logger = this.loggerResolver(this.configuration);

        if (this.configurationResolver && this.environment) {
            this.logger.debug(`Injecting configuration: ${this.configuration.constructor.name}`);
            this.injectionRegistry.replaceInjectable(this.configuration, [this.configuration.constructor]);
        }

        //-- inject logger
        this.logger.debug(`Injecting logger: ${this.logger.constructor.name}`);
        this.injectionRegistry.replaceInjectable(this.logger, [AbstractLogger]);

        this.logger.info(`Starting app, current environment: ${this.environment}`);
    }
}
